/* hi emacs, this is a part of -*- C -*- source */
/* $Id: bac_lexical.l,v 1.13 2002/12/30 17:25:04 oohara Exp $ */
/* process this file with flex -Pbac_ */

/* definitions */
%option noyywrap
%option nounput
%x value_string
%x value_long_int

%{
/* FILE, fdopen */
#include <stdio.h>
/* strtol */
#include <stdlib.h>
/* strdup */
#include <string.h>
/* pipe */
#include <unistd.h>
/* errno */
#include <errno.h>

/* this is generated by bison -d */
#include "bac_parser.h"

#include "bac_lexical.h"

static int line_number = 1;
static char *silly_tail = NULL;

%}

/* rules */
/* this comment must be before %% */
%%
[eE]ntry-[vV]ersion {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_ENTRY_VERSION;
  return BEGIN_ENTRY_VERSION;}

[cC]omment {
  BEGIN(value_string);
  bac_lval.val_int = BEGIN_COMMENT;
  return BEGIN_COMMENT;}

[sS]tage-[dD]ata-[vV]ersion {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_STAGE_DATA_VERSION;
  return BEGIN_STAGE_DATA_VERSION;}

[wW]hen {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_WHEN;
  return BEGIN_WHEN;}

[uU]id {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_UID;
  return BEGIN_UID;}

[uU]ser-[nN]ame {
  BEGIN(value_string);
  bac_lval.val_int = BEGIN_USER_NAME;
  return BEGIN_USER_NAME;}

[sS]core-[sS]ort {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_SCORE_SORT;
  return BEGIN_SCORE_SORT;}

[sS]core-[tT]otal {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_SCORE_TOTAL;
  return BEGIN_SCORE_TOTAL;}

[nN]umber-[sS]tage {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_NUMBER_STAGE;
  return BEGIN_NUMBER_STAGE;}

[sS]core-[sS]tage {
  BEGIN(value_long_int);
  bac_lval.val_int = BEGIN_SCORE_STAGE;
  return BEGIN_SCORE_STAGE;}

<value_long_int>[+\-]?[0-9]+ {
  /* you can write 7 as 007 for example, but it is always decimal,
   * not octal */
  /* to detect overflow */
  errno = 0;
  bac_lval.val_long_int = strtol(yytext, &silly_tail, 10);
  if (silly_tail[0] != '\0')
  {
    fprintf(stderr, "bac_lexical.l: strange integer\n");
    return LEXICAL_ERROR;
  }
  if (errno != 0)
  {
    fprintf(stderr, "bac_lexical.l: integer overflow\n");
    return LEXICAL_ERROR;
  }
  return LONG_INT;}

<value_string>[[:print:]]+ {
  /* note that [:print:] does not match \t or \n */
  bac_lval.val_string = strdup(yytext);
  return STRING;}

<value_long_int>\t |
<value_string>\t {
  bac_lval.val_int = '\t';
  return TAB;}

<value_long_int>\n |
<value_string>\n {
  BEGIN(INITIAL);
  bac_lval.val_int = '\n';
  line_number++;
  return NEWLINE;}

\n {
  /* empty line*/
  line_number++;
  return END_OF_ENTRY;}

\t {
  /* no field name */
  return LEXICAL_ERROR;}

<*>[^\n\t]+ {
  /* catch-all */
  return LEXICAL_ERROR;}

%%
/* user code */
int
bac_get_line_number(void)
{
  return line_number;
}

void
bac_clear_line_number(void)
{
  line_number = 1;
}

void
bac_read_from_file(FILE *file)
{
  /* yyrestart */
  bac_restart(file);
}

/* the return value is void because yyrestart() returns void
 * --- you can't check if it succeeded
 */
void
bac_read_from_string(const char *string)
{
  int filedes[2];
  FILE *stream[2];

  if (pipe(filedes) != 0)
  {
    fprintf(stderr, "pipe failed\n");
    return;
  }

  stream[0] = fdopen(filedes[0], "r");
  if (stream[0] == NULL)
  {
    fprintf(stderr, "fdopen(0) failed\n");
    return;
  }

  stream[1] = fdopen(filedes[1], "w");
  if (stream[1] == NULL)
  {
    fprintf(stderr, "fdopen(1) failed\n");
    return;
  }

  fprintf(stream[1], "%s", string);
  if (fflush(stream[1]) != 0)
  {
    fprintf(stderr, "fflush(1) failed\n");
    return;
  }

  /* The GNU libc manual says:
   * | A pipe or FIFO has to be open at both ends simultaneously.  If you
   * | read from a pipe or FIFO file that doesn't have any processes writing
   * | to it (perhaps because they have all closed the file, or exited), the
   * | read returns end-of-file.
   * I don't know why this fclose() is safe.
   */
  fclose(stream[1]);

  /* yyrestart */
  bac_restart(stream[0]);
}

void
bac_close_yyin(void)
{
  fclose(bac_in);
}
