/* $Id: bac_save.c,v 1.15 2003/01/02 19:24:33 oohara Exp $ */

#include <stdio.h>
/* memset, strlen */
#include <string.h>
/* free */
#include <stdlib.h>

/* libdb3 */
#include <db.h>

#include "bac_entry.h"
#include "bac_entry_to_string.h"

#include "bac_save.h"

/* return 0 on success, 1 on error
 * assumes dir (arg 1) already exists --- if not, it is an error
*/
int
bac_save(const char *dir, const char *file, bac_entry *p, int mode)
{
  DB *dbp = NULL;
  DB_ENV *dbenvp = NULL;
  DBT key;
  DBT data;

  /* sanity check */
  if (dir == NULL)
    return 0;
  if (file == NULL)
    return 0;
  if (p == NULL)
    return 0;
  if (p->klass != 0)
    return 0;

  /* this must be before db_create */
  if (db_env_create(&dbenvp, 0) != 0)
  {
    fprintf(stderr, "bac_save: db_env_create failed\n");
    return 1;
  }
  /* this must be before DB->open */
  /* this directory must be present on the system before
   * this program is called (DB_CREATE creates only files in that
   * directory, not the directory itself)
   */
  /* DB_INIT_CDB and DB_INIT_MPOOL means Concurrent Data Store */
  if (dbenvp->open(dbenvp, dir,
                   DB_INIT_CDB | DB_INIT_MPOOL | DB_CREATE, mode) != 0)
  {
    fprintf(stderr, "bac_save: DBENV->open(%s) failed\n", dir);
    dbenvp->close(dbenvp, 0);
    return 1;
  }
  if (db_create(&dbp, dbenvp, 0) != 0)
  {
    fprintf(stderr, "bac_save: db_create failed\n");
    dbenvp->close(dbenvp, 0);
    return 1;
  }
  if (dbp->open(dbp, file, NULL, DB_RECNO,
                DB_CREATE, mode) != 0)
  {
    fprintf(stderr, "bac_save: DB->open(%s) failed\n", file);
    dbp->close(dbp, 0);
    dbenvp->close(dbenvp, 0);
    return 1;
  }

  /* for compatibility with later version of libdb */
  memset(&key, 0, sizeof(key));
  memset(&data, 0, sizeof(data));
  /* we don't set key because this is a Recno database */
  data.data = bac_entry_to_string(p);
  /* I want to save the trailing \0 too */
  data.size = sizeof(char) * (strlen(data.data) + 1);
  if (dbp->put(dbp, NULL, &key, &data, DB_APPEND) != 0)
  {
    fprintf(stderr, "bac_save: DB->put failed\n");
    free(data.data);
    dbp->close(dbp, 0);
    dbenvp->close(dbenvp, 0);
    return 1;
  }

  free(data.data);
  if (dbp->close(dbp, 0) != 0)
  {
    dbenvp->close(dbenvp, 0);
    fprintf(stderr, "bac_save: DB->close failed\n");
    return 1;
  }
  if (dbenvp->close(dbenvp, 0) != 0)
  {
    fprintf(stderr, "bac_save: DBENV->close failed\n");
    return 1;
  }

  return 0;
}
